#!/usr/bin/python
""" 
	module gmsgui.py
		
    Copyright (C) 1997  George A. Farris
        george@gmsys.com - www.gmsys.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""
# Classes for gmsgui file
#
# attention - Takes a text message and displays it with a "Dismiss" button 
# alert     - Displays an alert box, must be destroyed manually
# dirSelect - Displays a directory listbox, ok button returns choosen directory
# listfile	- Displays contents of text file in text box.
#

import os
from Tkinter import *
from string import split
from ScrolledText import *
from posixpath import exists
from Dialog import *

#<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
# print text messages in the text box "intext"
# printText
#		- "message" is the text to print
#<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
class attention:
	def __init__(self,message):
		d = Dialog(None, title = 'Attention!',
				text = message,
				bitmap =  DIALOG_ICON,
				default = 0,
				strings = ('Dismiss',))

#<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
# Alert
#		Displays an alert with "message" on the screen
#<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
class alert:
	def __init__(self, message):
		self.top = Toplevel(None)
		self.top.title('Alert')
		self.top.iconname('Alert')
				
		self.bitmap = Label(self.top, bitmap='warning')
		self.bitmap.pack(side=LEFT, padx=4)
		self.message = Message(self.top, text=message, aspect=400)
		self.message.pack(side=LEFT, expand=1, fill=BOTH)

	def destroy(self):
		self.top.destroy()

#<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
# listfile
#		Displays a text file in a scrolledtext widget
#<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
class listfile:
	def __init__(self, file, manreader=None):
		if file[-1:] == '\n':	# chop any newlines, we hate em :-)
			file = file[:-1]

		gfs = os.popen('file '+file, 'r').read()	# check for man page
		for i in split(gfs):	# one of the words will be "troff" if it's a man page.
			if i == 'troff':
				if manreader:		# use a different man reader like tkman
					command = manreader+' '
				else:
					command = 'xterm -e man '
					
				try:
					os.system(command+file)		# display in xterm maybe make option
					return							# to use Tkman in future.
				except:
					attention("File Doesn't exist or is not readable")
					return
			else:
				try:
					fd = open(file, 'r')
				except:
					attention("File Doesn't exist or is not readable")
					return
		try:
			fd = open(file, 'r')
		except:
			attention("File Doesn't exist or is not readable")
			return
			
		self.top = Toplevel(None)
		self.top.title('List File')
		self.top.iconname('List')

		label= Label(self.top, 
				text="Listing of "+file,
				font='-adobe-helvetica-bold-r-normal--12-*-*-*-*-*-iso8859-1')
		label.pack(side=TOP)

		f1  = Frame(self.top, bd=5)							   # frame for scrolled text
		f1.pack(side=TOP, expand=YES, fill=BOTH)
		
		text = ScrolledText(f1, height=25, width=80)
		text.pack(side=LEFT, expand=YES, fill=BOTH)

		Button(self.top, text='Close', command=self.destroy, activebackground='grey90').pack(side=BOTTOM, padx=4, fill=X)

		for i in fd.read():
			text.insert('end', i)

	def destroy(self):
		self.top.destroy()

class copyMoveFile:

	def __init__(self, master, title, file):
		if title is None: title = self.title
		self.master = master					# give the instance a name
		self.init(master, title)				# draw all the gui stuff

		self.dirs.bind('<Double-1>', self.dirs_double_event) # bind double mouse click
		self.filter.bind('<Return>', self.mcd)		# cd to directroy
		
		self.ok_button = Button(self.botframe,	# setup the "OK" button
					 text="OK", width=10,padx=3,
					 command=self.ok_command)
		self.ok_button.pack(side=LEFT)			# pack it
		self.cancel_button = Button(self.botframe,	# setup the "Cancel" button
					    text="Cancel", width=10,padx=3,
					    command=self.cancel_command)
		self.cancel_button.pack(side=RIGHT)		# pack it
		self.top.protocol('WM_DELETE_WINDOW', self.cancel_command)

#************************************************************************************
# Init creates all the widgets for the window
# init(master, title)
#		- "master" is the root window id
#		- "title" is the title to put in the window bar
#------------------------------------------------------------------------------------
	def init(self, master, title):
		self.top = Toplevel(master)
		self.top.title(title)
		self.top.iconname(title)

		self.botframe = Frame(self.top, bd=3)		# frame for buttons
		self.botframe.pack(side=BOTTOM, fill=X)

		self.label1 = Label(self.top, text='Filter',
				anchor=W, font='-adobe-helvetica-bold-r-normal--12-*-*-*-*-*-iso8859-1')
		self.label1.pack(side=TOP, expand=YES, fill=X, padx=4)
		self.filter = Entry(self.top, bd=1, width=45)
		self.filter.pack(side=TOP, fill=X, padx=5)
		self.label2 = Label(self.top, text=' ')
		self.label2.pack(side=TOP)

		self.midframe = Frame(self.top,bd=2)		# frame for scrollbars
		self.midframe.pack(expand=YES, fill=BOTH, padx=4)


#************************************************************************************
# go(dir, default)
#		- "dir" id the directory to list
#		- "default" isn't used right now
#------------------------------------------------------------------------------------
	def go(self, dir=os.curdir, default=""):
		self.directory = os.path.expanduser(dir)
				
		self.load_command()
		self.how = None
		self.master.mainloop()		# Exited by self.quit(how)
		self.top.destroy()
		return self.how

	def quit(self, how=None):
		self.how = how
		self.master.quit()		# Exit mainloop()

#************************************************************************************
# go here if we manually enter directory in filter
# mcd()
#------------------------------------------------------------------------------------
	def mcd(self, event):
		self.directory = self.filter.get()
		self.load_command()
							
#************************************************************************************
# go here from the "OK" button
# ok_command()
#		- sort out what the directory is and then return to main program
#------------------------------------------------------------------------------------		
	def ok_command(self):
		index = self.dirs.curselection()     # fetch selection text	
		if index:							# if index is not empty
			label = self.dirs.get(index)     # "label" = selected directory
		else: 
			label = '.'						# else "label" = current directory
		if label == '..':					# if "label" = ".." choose rootdir which always points
			self.quit(self.rootdir)			# one directory back.
		else:								# else join the rootdir and the selected dir and quit
			self.quit(os.path.join(self.rootdir,label))

	def set_filter(self, dir):
		self.filter.delete(0, END)
		self.filter.insert(END, dir)

	def cancel_command(self, event=None):
		self.quit()

		


class dirSelect:

	title = "Directory Selection"

	def __init__(self, master, title=None):
		if title is None: title = self.title
		self.master = master					# give the instance a name
		self.rootdir = os.getcwd()				# get our current working directory
		self.directory = None
		self.createPMenu()						# create the popup menu fro right mouse button
		self.init(master, title)				# draw all the gui stuff
		self.dirs.bind('<Double-1>', self.dirs_double_event) # bind double mouse click
		self.dirs.bind('<ButtonPress-3>', self.cdmenupopup)         # set event handler		
		self.filter.bind('<Return>', self.mcd)		# cd to directroy
		
		self.ok_button = Button(self.botframe,	# setup the "OK" button
					 text="OK", width=10,padx=3,
					 command=self.ok_command)
		self.ok_button.pack(side=LEFT)			# pack it
		self.cancel_button = Button(self.botframe,	# setup the "Cancel" button
					    text="Cancel", width=10,padx=3,
					    command=self.cancel_command)
		self.cancel_button.pack(side=RIGHT)		# pack it
		self.top.protocol('WM_DELETE_WINDOW', self.cancel_command)

#************************************************************************************
# Init creates all the widgets for the window
# init(master, title)
#		- "master" is the root window id
#		- "title" is the title to put in the window bar
#------------------------------------------------------------------------------------
	def init(self, master, title):
		self.top = Toplevel(master)
		self.top.title(title)
		self.top.iconname(title)

		self.botframe = Frame(self.top, bd=3)		# frame for buttons
		self.botframe.pack(side=BOTTOM, fill=X)

		self.label1 = Label(self.top, text='Filter',
				anchor=W, font='-adobe-helvetica-bold-r-normal--12-*-*-*-*-*-iso8859-1')
		self.label1.pack(side=TOP, expand=YES, fill=X, padx=4)
		self.filter = Entry(self.top, bd=1, width=45)
		self.filter.pack(side=TOP, fill=X, padx=5)
		self.label2 = Label(self.top, text=' ')
		self.label2.pack(side=TOP)

		self.label = Label(self.top, text='Directory',
				anchor=W, font='-adobe-helvetica-bold-r-normal--12-*-*-*-*-*-iso8859-1')
		self.label.pack(side=TOP, expand=YES, fill=X, padx=4)
		

		self.midframe = Frame(self.top,bd=2)		# frame for scrollbars
		self.midframe.pack(expand=YES, fill=BOTH, padx=4)

		self.dirsbar = Scrollbar(self.midframe)
		self.dirsbar.pack(side=RIGHT, fill=Y)
		self.dirs = Listbox(self.midframe, exportselection=0,
				    width=25, yscrollcommand=(self.dirsbar, 'set'),
				    font='-adobe-courier-medium-r-normal--12-*-*-*-*-*-iso8859-1')
		self.dirs.pack(side=RIGHT, expand=YES, fill=BOTH)
		self.dirsbar.config(command=(self.dirs, 'yview'))

#************************************************************************************
# create a popup menu for the right mouse button
# createPMenu()
#------------------------------------------------------------------------------------
	def createPMenu(self):
		# popup for directory list box
		self.cdpopmenu = Menu(self.master, tearoff=0)
		self.cdpopmenu.add_command(label=' Home ', command=lambda x=self: x.load_command(os.environ['HOME']))
		self.cdpopmenu.add_command(label=' / ', command=lambda x=self: x.load_command('/'))
		self.cdpopmenu.add_command(label=' /usr ', command=lambda x=self: x.load_command('/usr'))
		self.cdpopmenu.add_command(label=' /tmp ', command=lambda x=self: x.load_command('/tmp'))



	def cdmenupopup(self, event): 
		self.cdpopmenu.tk_popup(self.master.winfo_pointerx(), self.master.winfo_pointery())

#************************************************************************************
# go(dir, default)
#		- "dir" id the directory to list
#		- "default" isn't used right now
#------------------------------------------------------------------------------------
	def go(self, dir=os.curdir, default=""):
		self.directory = os.path.expanduser(dir)
				
		self.load_command()
		self.how = None
		self.master.mainloop()		# Exited by self.quit(how)
		self.top.destroy()
		return self.how

	def quit(self, how=None):
		self.how = how
		self.master.quit()		# Exit mainloop()

#************************************************************************************
# dirs_double_event(event)
#		- "event"
#------------------------------------------------------------------------------------
	def dirs_double_event(self, event):
		index = self.dirs.curselection()     # fetch selection text	
		label = self.dirs.get(index)         # 
		self.directory = label
		self.load_command()

#************************************************************************************
# go here if we manually enter directory in filter
# mcd()
#------------------------------------------------------------------------------------
	def mcd(self, event):
		self.directory = self.filter.get()
		self.load_command()
							
#************************************************************************************
# go here from the "OK" button
# ok_command()
#		- sort out what the directory is and then return to main program
#------------------------------------------------------------------------------------		
	def ok_command(self):
		index = self.dirs.curselection()     # fetch selection text	
		if index:							# if index is not empty
			label = self.dirs.get(index)     # "label" = selected directory
		else: 
			label = '.'						# else "label" = current directory
		if label == '..':					# if "label" = ".." choose rootdir which always points
			self.quit(self.rootdir)			# one directory back.
		else:								# else join the rootdir and the selected dir and quit
			self.quit(os.path.join(self.rootdir,label))

#************************************************************************************
# go here from double click on directory
# load_command(dirs, files)
#		- "dirs" not used
#		- "files" not used
#		- reload the listbox with directories and files
#------------------------------------------------------------------------------------		
	def load_command(self, dirname=None, filename=None):
		if dirname == None:
			if self.directory == '..':
				self.rootdir, curdir = os.path.split(self.rootdir)
			elif self.directory == '.':
				self.rootdir == os.getcwd()
			else:
				self.rootdir = os.path.join(self.rootdir, self.directory)
		else:
			self.rootdir = dirname
			
		dirnames = os.listdir(self.rootdir)
		dirnames.sort()

		flist = os.listdir(self.rootdir)
		filenames = []
		for i in flist:
			if i[-4:] == '.rpm':
				filenames.append(i)

		filenames.sort()

		subdirs = ['..']

		for name in dirnames:
			fullname = os.path.join(self.rootdir, name)
			if os.path.isdir(fullname):
				subdirs.append(name)
			self.dirs.delete(0, END)
		for name in subdirs:
			self.dirs.insert(END, name)
		self.set_filter(self.rootdir)

	def set_filter(self, dir):
		self.filter.delete(0, END)
		self.filter.insert(END, dir)

	def cancel_command(self, event=None):
		self.quit()

		
#----------------------------------------------------------------------------------	
def testdir():
	"""Simple test program."""

	root = Tk()
	root.withdraw()
	dir = dirSelect(root)
	loadfile = dir.go()
	print loadfile

if __name__ == '__main__':
	testdir()
#	testftp()
