#!/usr/bin/python
""" 
	xrpm - version 2.03
		module gui.py
		
    Copyright (C) 1997  George A. Farris
        george@gmsys.com - www.gmsys.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""
INSTALLED = 'ins'
DIRECTORY = 'dir'
GROUP     = 'grp'
FTP       = 'ftp'


import os
from Tkinter import *
from ftputil import ftpsites
from string import split
from posixpath import exists
from Dialog import *
from gmsgui import dirSelect, listfile

class GetDir:

	title = "Directory Selection"

	def __init__(self, master, title=None):
		if title is None: title = self.title
		self.master = master					# give the instance a name
		self.id = self.__class__.__name__		# see if this is GetDir or GetFtp
		self.rootdir = os.getcwd()				# get our current working directory
		self.directory = None
		self.createPMenu()						# create the popup menu fro right mouse button
		self.init(master, title)				# draw all the gui stuff

		self.dirs.bind('<Double-1>', self.dirs_double_event) # bind double mouse click
		self.dirs.bind('<ButtonPress-3>', self.cdmenupopup)         # set event handler		
		self.files.bind('<ButtonPress-3>', self.menupopup)         # set event handler
		self.filter.bind('<Return>', self.mcd)		# cd to directroy
		
		self.ok_button = Button(self.botframe,	# setup the "OK" button
					 text="OK", width=10,padx=3,
					 command=self.ok_command)
		self.ok_button.pack(side=LEFT)			# pack it
		self.cancel_button = Button(self.botframe,	# setup the "Cancel" button
					    text="Cancel", width=10,padx=3,
					    command=self.cancel_command)
		self.cancel_button.pack(side=RIGHT)		# pack it
		self.top.protocol('WM_DELETE_WINDOW', self.cancel_command)

#************************************************************************************
# Init creates all the widgets for the window
# init(master, title)
#		- "master" is the root window id
#		- "title" is the title to put in the window bar
#------------------------------------------------------------------------------------
	def init(self, master, title):
		self.top = Toplevel(master)
		self.top.title(title)
		self.top.iconname(title)

		self.botframe = Frame(self.top, bd=3)		# frame for buttons
		self.botframe.pack(side=BOTTOM, fill=X)

		self.label1 = Label(self.top, text='Filter',
				anchor=W, font='-adobe-helvetica-bold-r-normal--12-*-*-*-*-*-iso8859-1')
		self.label1.pack(side=TOP, fill=X, padx=4)
		self.filter = Entry(self.top, bd=1)
		self.filter.pack(side=TOP, fill=X, padx=5)

		if self.id == 'GetFtp':		# Put the checkbutton for user id here
			self.fcbpwd= Checkbutton(self.top, command=self.fcbpromptset,
				   text='Prompt for user id and password.', anchor=W)
			self.fcbpwd.pack(side=TOP, padx=3, pady=5, fill=X)

		self.label2 = Label(self.top, text=' ')
		self.label2.pack(side=TOP)

		self.label = Label(self.top, text='Directory                                    RPM''s Available ',
				anchor=W, font='-adobe-helvetica-bold-r-normal--12-*-*-*-*-*-iso8859-1')
		self.label.pack(side=TOP, fill=X, padx=4)
		

		self.midframe = Frame(self.top,bd=2)		# frame for scrollbars
		self.midframe.pack(expand=YES, fill=BOTH, padx=4)

		self.filesbar = Scrollbar(self.midframe)
		self.filesbar.pack(side=RIGHT, fill=Y)
		self.files = Listbox(self.midframe, exportselection=0,
				width=30, height=15, yscrollcommand=(self.filesbar, 'set'),
				font='-adobe-courier-medium-r-normal--12-*-*-*-*-*-iso8859-1')
		self.files.pack(side=RIGHT, expand=YES, fill=BOTH)
		self.filesbar.config(command=(self.files, 'yview'))
		
		self.dirsbar = Scrollbar(self.midframe)
		self.dirsbar.pack(side=RIGHT, fill=Y)
		self.dirs = Listbox(self.midframe, exportselection=0,
				    width=25, yscrollcommand=(self.dirsbar, 'set'),
				    font='-adobe-courier-medium-r-normal--12-*-*-*-*-*-iso8859-1')
		self.dirs.pack(side=RIGHT, expand=YES, fill=BOTH)
		self.dirsbar.config(command=(self.dirs, 'yview'))

#************************************************************************************
# create a popup menu for the right mouse button
# createPMenu()
#------------------------------------------------------------------------------------
	def createPMenu(self):
		# popup for file listbox
		self.popmenu = Menu(self.master, tearoff=0)
		self.popmenu.add_command(label=' Copy ', command=self.copy)
		self.popmenu.add_command(label=' Move ', command=self.move)
		self.popmenu.add_command(label=' Delete ', command=self.delete)

		# popup for directory list box
		self.cdpopmenu = Menu(self.master, tearoff=0)
		self.cdpopmenu.add_command(label=' Home ', command=lambda x=self: x.load_command(os.environ['HOME']))
		self.cdpopmenu.add_command(label=' / ', command=lambda x=self: x.load_command('/'))
		self.cdpopmenu.add_command(label=' /usr ', command=lambda x=self: x.load_command('/usr'))
		self.cdpopmenu.add_command(label=' /tmp ', command=lambda x=self: x.load_command('/tmp'))
		self.cdpopmenu.add_command(label=' /mnt/cdrom ', command=lambda x=self: x.load_command('/mnt/cdrom'))
				
	def copy(self):
		index = self.files.curselection()     # fetch selection text	
		label = self.files.get(index)         # 

		top = Toplevel(None)
		top.withdraw()
		dir = dirSelect(top)
		destination = dir.go()
		source = os.path.join(self.rootdir,label)
		os.system('cp %s %s' % (source,destination))

	
	def move(self):
		index = self.files.curselection()     # fetch selection text	
		label = self.files.get(index)         # 

		top = Toplevel(None)
		top.withdraw()
		dir = dirSelect(top)
		destination = dir.go()
		source = os.path.join(self.rootdir,label)
		os.system('cp %s %s' % (source,destination))
		os.unlink(source)  # delete it
		self.load_command('.')  # reload the listbox without file
	
	def delete(self):
		index = self.files.curselection()     # fetch selection text	
		label = self.files.get(index)         # 
		d = Dialog(None, title = 'Attention!',
			text = 'You have requested this package be deleted!\n\n'
					'Please confirm or cancel.',
			bitmap =  DIALOG_ICON,
			default = 0,
			strings = ('Delete','Cancel'))

		if d.num == 0:
			os.unlink(os.path.join(self.rootdir, label))  # delete it
			self.load_command('.')  # reload the listbox without file

		
	def menupopup(self, event): 
		index = self.files.curselection()
		label = self.files.get(index)
		self.popmenu.tk_popup(self.master.winfo_pointerx(), self.master.winfo_pointery())
	
	def cdmenupopup(self, event): 
#		index = self.files.curselection()
#		label = self.files.get(index)
		self.cdpopmenu.tk_popup(self.master.winfo_pointerx(), self.master.winfo_pointery())

#************************************************************************************
# go(dir, default)
#		- "dir" id the directory to list
#		- "default" isn't used right now
#------------------------------------------------------------------------------------
	def go(self, dir=os.curdir, default=""):
		if self.id == 'GetDir': 
			self.directory = os.path.expanduser(dir)
				
		self.load_command()
		self.how = None
		self.master.mainloop()		# Exited by self.quit(how)
		self.top.destroy()
		return self.how

	def quit(self, how=None):
		self.how = how
		self.master.quit()		# Exit mainloop()

#************************************************************************************
# dirs_double_event(event)
#		- "event"
#------------------------------------------------------------------------------------
	def dirs_double_event(self, event):
		index = self.dirs.curselection()     # fetch selection text	
		label = self.dirs.get(index)         # 
		if self.id == 'GetDir':
			self.directory = label
			self.load_command()
		elif self.id == 'GetFtp':
			self.connected_site = label
			self.opensite(label,index)

#************************************************************************************
# go here if we manually enter directory in filter
# mcd()
#------------------------------------------------------------------------------------
	def mcd(self, event):
		self.directory = self.filter.get()
		self.load_command()
							
#************************************************************************************
# go here from the "OK" button
# ok_command()
#		- sort out what the directory is and then return to main program
#------------------------------------------------------------------------------------		
	def ok_command(self):
		index = self.dirs.curselection()     # fetch selection text	
		if index:							# if index is not empty
			label = self.dirs.get(index)     # "label" = selected directory
		else: 
			label = '.'						# else "label" = current directory

		if label == '..':					# if "label" = ".." choose rootdir which always points
			self.quit(self.rootdir)			# one directory back.
		else:								# else join the rootdir and the selected dir and quit
			self.quit(os.path.join(self.rootdir,label))

#************************************************************************************
# go here from double click on directory
# load_command(dirs, files)
#		- "dirs" not used
#		- "files" not used
#		- reload the listbox with directories and files
#------------------------------------------------------------------------------------		
	def load_command(self, dirname=None, filename=None):
		if dirname == None:
			if self.directory == '..':
				self.rootdir, curdir = os.path.split(self.rootdir)
			elif self.directory == '.':
				self.rootdir == os.getcwd()
			else:
				self.rootdir = os.path.join(self.rootdir, self.directory)
		else:
			self.rootdir = dirname
			
		dirnames = os.listdir(self.rootdir)
		dirnames.sort()

		flist = os.listdir(self.rootdir)
		filenames = []
		for i in flist:
			if i[-4:] == '.rpm':
				filenames.append(i)

		filenames.sort()

		subdirs = ['..']

		for name in dirnames:
			fullname = os.path.join(self.rootdir, name)
			if os.path.isdir(fullname):
				subdirs.append(name)
			self.dirs.delete(0, END)
		for name in subdirs:
			self.dirs.insert(END, name)
		self.set_filter(self.rootdir)

		self.files.delete(0, END)
		pos = 0
		for package in filenames:    # add to list-box
			self.files.insert(pos, os.path.basename(package))  # and menu/toolbars
			pos = pos + 1

	def set_filter(self, dir):
		self.filter.delete(0, END)
		self.filter.insert(END, dir)

	def cancel_command(self, event=None):
		self.quit()

		




###############################################################################
#
# Here we get the directory and rpms of the ftp site.  
# After that everything is the same as looking at the local directory
# well almost :-)
#
###############################################################################
class GetFtp(GetDir, ftpsites):

	title = "Ftp Sites"
	
	def __init__(self, master, title=None):
		if title is None: title = self.title
		self.master = master
		self.id = self.__class__.__name__
		
		self.init(master, title)		# Do most of the list init
		
		self.fcbprompt = 0		# checkbutton prompt for user id and password
		
		self.dirs.bind('<Double-1>', self.dirs_double_event)	# Bind this to open the ftp site
		self.files.bind('<Double-1>', self.files_double_event)	# Bind this to open the ftp site
		self.filter.bind('<Return>', self.mopensite)		# Enter site and open

		self.label.config(text='FTP Site                                 Directory to open')

		self.files.config(width=50)		# Lets stretch the file list
		self.ok_button = Button(self.botframe,text="Open",
				width=10,padx=3,command=self.ok_opensite)
		self.ok_button.pack(side=LEFT)

		self.store_button = Button(self.botframe,text="Save Site",
				width=10,padx=3,command=self.saveFtpSite, state='disabled')
		self.store_button.pack(side=LEFT)

		self.counter = Label(self.botframe, text='            You may open the SITE or the DIRECTORY')	# set up label for a line counter
		self.counter.pack(side=LEFT)					# which gives us visual feedback when
														# getting dir listings.

		self.cancel_button = Button(self.botframe,
				width=10,padx=3,text="Cancel",command=self.cancel_command)
		self.cancel_button.pack(side=RIGHT)
		self.top.protocol('WM_DELETE_WINDOW', self.cancel_command)

		self.ftplist = ftpsites()		# make an instance of ftpsites()

		# here we open our own ".ftp-sites" file or if it doesn't exist the
		# system wide one.
		homeDir = os.environ['HOME']
		if exists(homeDir+'/.ftp-sites'):
			ftpfile = self.ftplist.readlines(homeDir+'/.ftp-sites')	# Get our stored ftp list
			self.top.title('Ftp Sites - '+homeDir+'/.ftp-sites')
		elif exists('/usr/lib/xrpm/ftp-sites'):
			ftpfile = self.ftplist.readlines('/usr/lib/xrpm/ftp-sites')
			self.top.title('Ftp Sites - /usr/lib/xrpm/ftp-sites')
		else:
			d = Dialog(None, {'title': 'File Error!',
					'text':
					'The ftp-sites file is missing'
					' corrupt.'
					' Check the file and try again.',
					'bitmap': DIALOG_ICON,
					'default': 0,
					'strings': ('Dismiss',)})
			self.quit(None)
																
		fsite = fpath = []
		self.ftpFileDict = {}

		self.ftpFileEntryCount = 0
		
		for line in ftpfile:			# we need to split each line in the file into
			s_tmp = split(line)			# site / path pairs and store them in dictionary
			fsite = fsite + [s_tmp[0]]
			fpath = fpath + [s_tmp[1]]
			self.ftpFileDict[self.ftpFileEntryCount] = [s_tmp[0], s_tmp[1]]	# store site and path in dictionary
			self.ftpFileEntryCount = self.ftpFileEntryCount + 1				# for later use when opening ftp site
		
		self.ftpFileEntryCount = self.ftpFileEntryCount - 1		# actual count is one less
		
		# Set up the listboxes for dual scroll with one scrollbar
		self.files.config(yscrollcommand=(self.filesbar, 'set'))
		self.dirs.config(yscrollcommand=(self.filesbar, 'set'))
		self.filesbar.config(command=(self.bindyview))
		self.load_command(fsite, fpath)	# First pass through the listbox load the file
		Pack.forget(self.dirsbar)	# Hide the dir scroll bar for now

	def fcbpromptset(self):
		if self.fcbprompt == 0:
			self.fcbprompt = 1
		else:
			self.fcbprompt = 0
		
# This is the double listbox scroll command
	def bindyview(self, a=None, b=None, c=None):
		self.dirs.yview(a, b, c)
		self.files.yview(a, b, c)
				
	def load_command(self, dirs=None, files=None):
		
		if dirs:						# Clear and load the listbox only if new data
			self.dirs.delete(0, END)	
			for name in dirs:
				self.dirs.insert(END, name)
				
		if files: 						# Clear and load the listbox only if new data
			self.files.delete(0, END)		# clear file list no matter what
			for names in files:
				self.files.insert(END, names)
				
		self.set_filter('')	# Set the entry line with the name of the host.

#************************************************************************************
# if we select the path instead of the site we end up here
# files_double_event(event)
#		- "event"
#------------------------------------------------------------------------------------
	def files_double_event(self, event):
		index = self.files.curselection()     # fetch selection text	
		label = self.files.get(index)         # 
		self.connected_site = self.ftpFileDict[eval(index[0])][0]
		self.opensite(label,index)
		
#************************************************************************************
# Go here if we enter a site name manually in the entry widget
# mopensite()
#------------------------------------------------------------------------------------
	def mopensite(self, event):
		self.connected_site = self.filter.get()
		self.opensite(self.connected_site, -1)	# -1 is no index we test for this below

#************************************************************************************
# Go here if we hit the "OK" button
# ok_opensite()
#------------------------------------------------------------------------------------
	def ok_opensite(self):

		try:
			index = self.dirs.curselection()     # fetch selection text
			label = self.dirs.get(index)  		# label is the actual text of the selection
		except:
			try:
				index = self.files.curselection()     # fetch selection text	
				label = self.files.get(index)         # 
			except:
				return		# return if open button pressed but no selection
				
		self.connected_site = self.ftpFileDict[eval(index[0])][0]
		self.opensite(label,index)


#************************************************************************************
# Go here from double_event or ok_opensite which is really the "OK" button
# opensite(s)
#		- "s" is the ftp site to open
#------------------------------------------------------------------------------------
	def opensite(self, ftpdict, dictIndex):

		if dictIndex == -1:
			site = ftpdict			# if it's a manual entry then this is the site
		else:
			site = self.ftpFileDict[eval(dictIndex[0])][0]

		self.midframe.config(cursor='watch')	# do this to give us a watch cursor
		self.master.update()					# while waiting.

		self.uid = None				# set initial value for test in if below
		self.passwd = None

		if self.fcbprompt == 1:				# if we ask for user id go here
			self.dirs.bind('<Double-1>', '')	# Bind this to open the ftp site
			self.files.bind('<Double-1>', '')	# Bind this to open the ftp site
			self.filter.bind('<Return>', '')		# Enter site and open
			self.ok_button.config(command='')
			
			self.getUidInfo(site)		# get user id and password

			self.dirs.bind('<Double-1>', self.dirs_double_event)	# Bind this to open the ftp site
			self.files.bind('<Double-1>', self.files_double_event)	# Bind this to open the ftp site
			self.filter.bind('<Return>', self.mopensite)		# Enter site and open
			self.ok_button.config(command=self.ok_opensite)

			if self.uid == None:		# if nothing entered go back to choosing site
				self.midframe.config(cursor='')		# remove watch cursor
				self.master.update()
				return
			
		try:
			self.site = self.ftplist.open(site, self.uid, self.passwd)	# open the ftp site and login anonymously
		except:
			self.midframe.config(cursor='')		
			self.master.update()

			d = Dialog(None, {'title': 'Network Error!',
					'text':
					'The FTP site you were attempting'
					' to connect to returned an error.'
					' Check the host name or try again'
					' later.',
					'bitmap': DIALOG_ICON,
					'default': 0,
					'strings': ('Dismiss',)})

		else:		
			# restore the listboxes for single scroll
			Pack.forget(self.fcbpwd)  # hide the check button
			Pack.forget(self.dirs)	# Hide the dir so we can repack properly
			self.dirsbar.pack(side=RIGHT, fill=Y)
			self.dirs.pack(side=RIGHT, expand=YES, fill=BOTH)
	
			self.store_button.config(state='active')	
	
			self.dirs.config(yscrollcommand=(self.dirsbar, 'set'))
			self.filesbar.config(command=(self.files, 'yview'))
			self.dirs.config(width=20)		# Lets shrink the dir list

			# Rebind these for site that is already open.
			self.dirs.bind('<Double-1>', self.doubleClick_ftpcwd)	# double click changes dir.
			self.filter.bind('<Return>', self.mftpcwd)	# or enter it manually and <enter> 
			self.files.bind('<Double-1>', self.listFtpFile)	# Bind this to open the ftp site
			
			self.ok_button.config(command=self.ok_command, text="OK")
			self.label.config(text='Directory                            RPM''s Available ')

			if dictIndex != -1:
				if self.ftpFileDict[ eval(dictIndex[0]) ][0] == ftpdict: 
					if self.ftpcwd('/')	== 'error':		# cd to the root
						self.top.destroy()
						self.mode = INSTALLED
						self.uid = None		# clear any uid and passwd from an FTP session
						self.passwd = None
						return
				else:
					if self.ftpcwd(self.ftpFileDict[eval(dictIndex[0])][1])	== 'error': # or to the stored path
						self.top.destroy()
						self.mode = INSTALLED
						self.uid = None		# clear any uid and passwd from an FTP session
						self.passwd = None
						return
			else:
				self.ftpcwd('/')
					
			self.midframe.config(cursor='')		
			self.master.update()

#************************************************************************************
# get user id and password
#	- site is the site to connect to
#------------------------------------------------------------------------------------
	def getUidInfo(self, site):
		self.ftpLogin = Toplevel(None)

		ff = Frame(self.ftpLogin, bd=2, relief=RIDGE)
		ff.pack(side=TOP, padx=15, pady=15)
			
		uid = Frame(ff)
		uid.pack(side=TOP, fill=X, expand=YES, pady=4)
		pwd = Frame(ff)
		pwd.pack(side=TOP, fill=X, expand=YES, pady=4)
			
		Label(uid, text='Login:').pack(side=LEFT)
		self.userid = Entry(uid, bd=1)
		self.userid.pack(side=RIGHT, fill=X, padx=5)
		
		Label(pwd, text='Password:').pack(side=LEFT)		
		self.passwd = Entry(pwd, bd=1, show='*')
		self.passwd.pack(side=RIGHT, fill=X, padx=5)
		self.passwd.bind('<Return>', self.getftpentry)		# Enter site and open

		self.ftpLogin.title('User Login')
		
		Button(self.ftpLogin, text='OK',
				command=self.getftpentry, activebackground='grey90').pack(side=TOP, padx=4, expand=YES, fill=X)

		self.ftpLogin.wait_window()

	def getftpentry(self, event=None):
		if event:
			self.uid = self.userid.get()
			self.passwd = self.passwd.get()
			self.ftpLogin.destroy()
		else:
			self.ftpLogin.destroy()


#************************************************************************************
# Get a text file from an FTP site and display it.
# listFtpFile(event)
#		- "event" event from double click in file box.
#------------------------------------------------------------------------------------
	def listFtpFile(self, event):
		index = self.files.curselection()     # fetch selection text	
		label = self.files.get(index)         # 
		filename = split(label)[4]
		self.ftpget(self.ftpdir, filename)
		listfile('/tmp/'+filename)
		os.unlink('/tmp/'+filename)				

#************************************************************************************
# Dig out directories, files and sizes.  If ext = rpm, get a list of rpm files only.
# format_dir(line,ext)
#		- "line" a list of all the files in the directory
#		- "ext"
#------------------------------------------------------------------------------------
	def format_dir(self, line, ext=None):
		dirs = []
		files = []
		gg = ''
		for i in line:
			s = split(i)
			dorf = s[0]		# get first chunk drwxr-xr-x
			if dorf[0] == 'd':	# get directories
				dirs.append(s[-1])

			elif dorf[0] == '-':
				# if we're looking for just rpm files this is where it happens
				if ext == 'rpm':
					if i[-4:] == '.rpm':
						all =('ftp:/%s/%s' % (self.ftpdir, s[-1]))
						files.append(all)
				else:
					# format the files list
					all = ('%8s %3s %2s %5s %s' % (s[-5],s[-4],s[-3],s[-2],s[-1]))
					files.append(all)	
				
		if files == []:		# if files is still empty make sure listbox is clear
			files = ''
		return dirs, files

#************************************************************************************
# we go here with double click once site is open.
#------------------------------------------------------------------------------------
	def doubleClick_ftpcwd(self, event):
		index = self.dirs.curselection()	# fetch selection text	
		label = self.dirs.get(index)         
		self.ftpcwd(label)		# change directory
		
#************************************************************************************
# we go here if a directory is entered in the top line
#------------------------------------------------------------------------------------
	def mftpcwd(self, event):
		s = self.filter.get()
		self.ftpcwd(s)			# change directory
		
#************************************************************************************		
# this is the actual change directory function.
#------------------------------------------------------------------------------------
	def ftpcwd(self,dir):
		self.midframe.config(cursor='watch')		
		self.files.delete(0, END)		# clear file list no matter what
		self.master.update()

		try:
			self.site.cwd(dir)				# change the directory
		except:
			d = Dialog(None, {'title': 'Network Error!',
					'text':
					'The directory on the FTP site you were attempting'
					' to change to returned an error. Check the'
					' host name and permissions of your account.'
					' Disconnecting!!!.',
					'bitmap': DIALOG_ICON,
					'default': 0,
					'strings': ('Dismiss',)})
			self.midframe.config(cursor='')		
			self.master.update()
			return 'error'
						
		self.ftpdir = self.site.pwd()		# save the current directory
		self.ftpcwdCounter = 1
		self.ftpcwdLines = []

		self.site.retrlines('LIST', self.ftpcwdCB)	# run a new dir command
		dirs, self.ftpFileList = self.format_dir(self.ftpcwdLines) # format it for display 
		self.load_command(dirs, self.ftpFileList)		# 
		self.set_filter(self.ftpdir)

		self.midframe.config(cursor='')		

		self.master.update()

#************************************************************************************
# this is a callback for the ftp directory listing.  it just gives us a visual
# indication that something is happening by counting the lines retrieved and 
# displaying them in a label
#------------------------------------------------------------------------------------
	def ftpcwdCB(self,line):
		self.ftpcwdLines.append(line)
		self.counter.config(text='                         Reading line: '+`self.ftpcwdCounter`)
		self.master.update()
		self.ftpcwdCounter = self.ftpcwdCounter + 1

#************************************************************************************
# This is where we actually get the package from the ftp site.  Pop up a scale bar
# and show transfer progress.  At least we can see something happening.
# ftpget(path, file)
#		- "path" is the path on the ftp server
#		- "file" is the name of the file we're downloading.
#------------------------------------------------------------------------------------
	def ftpget(self, path, file):
		global ftpfd		# make these global so the callback can access them
		
		self.ftpfilename = file
		
		fsc = Toplevel(self.master)
		fsc.title('Retrieving RPM')
		fsc.sc = Frame(fsc, bd=2, relief=RAISED)
		fsc.sc.pack(side=TOP, padx=5, pady=7)
		label = Label(fsc.sc, text='Retrieving '+file)
		label.pack(side=TOP)
		self.ftpScale = Scale(fsc.sc, relief=RAISED,orient = 'horizontal', length='10c')
		self.ftpScale.pack(side=TOP, padx=5, pady=7)
				
		Button(fsc, text='Abort', 
				command=self.site.abort, activebackground='grey90').pack(side=TOP, padx=4, expand=YES, fill=X)

		ftpfd = open('/tmp/'+file, 'w')
		self.ftpfilesize = self.site.size(path+'/'+file)
		self.site.retrbinary('RETR ' + path +'/'+ file, self.ftpgetCB, 8192)
		ftpfd.close()
		fsc.destroy()
		
	def ftpgetCB(self, data):
		size = os.stat('/tmp/'+self.ftpfilename)[6]
		tickcount = int((size / float(self.ftpfilesize))*100)
		self.ftpScale.set(tickcount)
		ftpfd.write(data)		
		self.master.update()
		
#************************************************************************************
# go here once we have an ftp directory that we want, then get a list 
# of all the rpm's and return it.
#------------------------------------------------------------------------------------
	def ok_command(self):
		files = []
		for i in self.ftpFileList:
			s = split(i)				# split the line at white space
			if s[-1][-4:] == '.rpm':
				files.append(s[-1])		# get the last item on the line which is the
										# filename
		self.counter.config(text=' ')	# Remove the line counter										
		self.quit(files)
						
#----------------------------------------------------------------------------------	
def testdir():
	"""Simple test program."""

	root = Tk()
	root.withdraw()
	dir = GetDir(root)
	loadfile = dir.go()
	print loadfile

def testftp():
	root = Tk()
	root.withdraw()
	dir = GetFtp(root)
	loadfile = dir.go()
	print loadfile

if __name__ == '__main__':
	testdir()
#	testftp()
