#!/usr/bin/python

""" 
	xrpm - version 2.1
		module rpm.py
		
    Copyright (C) 1997  George A. Farris
        george@gmsys.com - www.gmsys.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""
INSTALLED = 'ins'
DIRECTORY = 'dir'
GROUP     = 'grp'
FTP       = 'ftp'
COMMAND   = 'com'	# called from command line
MANREADER = 'tkman -now'	# man page reader

import os
import ftplib
from gui import GetFtp
from Tkinter import *
from ScrolledText import *
from string import rfind, atoi, split
from stat import *
from Dialog import *
from gmsgui import alert, attention, listfile

class Rpm:
#************************************************************************************
# listFiles(url)
#		- "url" is a regular unix path or a ftp url
#		- returns a list of files in an rpm
#		- list could be installed, uninstalled or rpm's from an ftp site
#------------------------------------------------------------------------------------
	def listFiles(self, url):
		if url[:6] == 'ftp://':
			rpmlist = os.popen('rpm -qpl %s' % (url), 'r').readlines()
		elif url[:1] == '/':
			rpmlist = os.popen('rpm -qpl %s' % (url), 'r').readlines()
		else:
			# It should be an installed rpm
			rpmlist = os.popen('rpm -ql %s' % (url), 'r').readlines()
		return rpmlist

#************************************************************************************
# displayFiles(label)
#		- "index" is the index in the main listbox of selected rpm's
#		- opens a lisbox and displays the files contained in the rpm(s)
#		- "file" is from command line call
#------------------------------------------------------------------------------------
	def displayFiles(self, index, listbox, file=None):
		self.flbindex = index		# have to make these global for tk, blahhhh!!
		self.flbindex_len = 0

		flb = Toplevel(self)
		self.flb = flb				# do this so we can kill it from xrpm.py in the
									# case of a command line call.
		flb.title('List Package Files')

		self.dflabel = Label(flb, text = ' ')
		self.dflabel.pack(side=TOP)

		dfframe  = Frame(flb, bd=3)
		dfframe.pack(side=TOP, expand=YES, fill=BOTH, padx=5, pady=7)
		
		dfscroll = Scrollbar(dfframe, relief=SUNKEN)
		self.dflist   = Listbox(dfframe, height=20, width=70,
				font='-adobe-courier-medium-r-normal--12-*-*-*-*-*-iso8859-1' )

		self.dflist.config(yscrollcommand=dfscroll.set)
		self.dflist.pack(side=LEFT, expand=YES, fill=BOTH)
		
#		if self.mode == INSTALLED or self.mode == GROUP:		# if we're in either of these two modes
		self.dflist.bind('<Double-1>', self.double1Event)	# we can look at a text file.

		dfscroll.config(command=self.dflist.yview, relief=SUNKEN)
		dfscroll.pack(side=RIGHT, fill=BOTH)

		if len(self.flbindex) > 1:	# If we have multiple selections
			self.flbbut = Button(flb, text='Next Package',command=lambda x=self: x.reload_filebox(x.listbox, 'Y'), 
					activebackground='grey90')
			self.flbbut.pack(side=TOP, padx=4, fill=X)

		Button(flb, text='Close', command=flb.destroy, activebackground='grey90').pack(side=BOTTOM, padx=4, fill=X)
		
		self.reload_filebox(listbox, 'N', file)


#************************************************************************************
# double1Event(event)
#	Go here from double click in list box that shows files inside a package
#------------------------------------------------------------------------------------
	def double1Event(self, event):
		index = self.dflist.curselection()     # fetch selection text	
		filename = self.dflist.get(index)
		
		if self.mode == INSTALLED or self.mode == GROUP:		# if we're in either of these two modes
			self.dflist.config(cursor='watch')
			self.master.update()

			if self.ConfigUseTkman.get():
				listfile(filename, MANREADER)
			else:
				listfile(filename)

			self.dflist.config(cursor='')

		if self.mode == DIRECTORY:
			self.dflist.config(cursor='watch')
			self.master.update()
			filename = filename[1:]		# strip leading '/'
			curdir = os.getcwd()		# save our current dir
			os.chdir('/tmp')			# make our working directory /tmp
			command = 'rpm2cpio '+self.selectedPackage+' | cpio -i --make-directories '+filename
			os.system(command)			# run the command

			if filename[-1:] == '/n':
				filename = filename[:-1]
				
#			gfs = os.lstat('/tmp/'+filename)		# stupid lstat dosen't work?????

			gfs = os.popen('file '+filename, 'r').read()	# use my hack instead
			for i in split(gfs):
				if i == 'symbolic':
					attention('This is a symbolic link')
					s = split(filename, '/')	# split the filename for the path
					os.system('rm -rf '+s[0])	# recursively delete the file
					os.chdir(curdir)			# restore our working dir
					self.dflist.config(cursor='')
					return

			self.dflist.config(cursor='')
			if self.ConfigUseTkman.get():
				listfile(filename, MANREADER)
			else:
				listfile(filename)
			s = split(filename, '/')	# split the filename for the path
			os.system('rm -rf '+s[0])	# recursively delete the file
			os.chdir(curdir)			# restore our working dir
		
		if self.mode == FTP:
			d = Dialog(None, title = 'Attention!',
				text = "Reading files from inside a package from an FTP site isn't"
					" supported at the moment.  If you really want this feature email"
					" me and I'll add it if I get enough responses.\n\n"
					"Note that the entire package will have to be transferred BEFORE "
					" the file can be read.", 
#				text = 'This package is from an FTP site.  You will have to'
#					' download the entire RPM file to view the file you selected.',
				bitmap =  DIALOG_ICON,
				default = 0,
#				default = 1,
				strings = ('Dismiss',))
#				strings = ('Continue', 'Cancel'))
#			if d.num == 0:
#				print 'Do ftp stuff'
		

#************************************************************************************
# reload_filebox(listbox,multi,file)
#	
#------------------------------------------------------------------------------------
	def reload_filebox(self,listbox, multi=None, file=None):
		if self.flbindex_len == len(self.flbindex):	# If we've looked at all the selecetions
			return									# return
		if multi == 'Y':
			if self.flbindex_len == len(self.flbindex)-1:
				Pack.forget(self.flbbut)				# loose this button for the last entry

		if not file:
			label = listbox.get(self.flbindex[self.flbindex_len])
		else:
			label = file

		self.dflabel.config(text=label)
				
		if self.mode == INSTALLED or self.mode == GROUP:					# if newdir is ""None"" it's installed rpm
			s = label								# display it
		elif self.mode == FTP:						# if called from getftpsite()
			if self.con.uid:		# if there is a user id
				s = 'ftp://%s:%s@%s%s/%s' % (self.con.uid, self.con.passwd,self.con.connected_site, self.con.ftpdir, label)
			else:
				s = 'ftp://%s%s/%s' % (self.con.connected_site, self.con.ftpdir, label)
		else:									# it's a directory query so	
			s = os.path.join(self.newdir, label)	# join the root dir and the cwd
		
		self.selectedPackage = s	# do this for reading a file from an uninstalled rpm
									# we use it in "double1Event above
		
		if not file:
			self.mlbframe.config(cursor='watch')	# do this to give us a watch cursor
			self.master.update()					# while waiting.
			nn = self.listFiles(s)
			self.mlbframe.config(cursor='')		
			self.master.update()
			self.flbindex_len = self.flbindex_len + 1		# increment our selection counter
		else:
			nn = self.listFiles(s)

		self.dflist.delete(0, END)
		pos = 0
		for label in nn:    # add to list-box
			if label[-1:] == '\n':
				label == label[:-1]
			self.dflist.insert(pos, label)                      # and menu/toolbars
			pos = pos + 1
			

#************************************************************************************
# query(url)
#		- "url" is a regular unix path or a ftp url
#		- returns a text query of rpm
#		- query could be installed, uninstalled or rpm's from an ftp site
#------------------------------------------------------------------------------------
	def query(self, url):
		if url[:6] == 'ftp://':
			rpmlist = os.popen('rpm -qpi %s' % (url), 'r').read()		
		elif url[:1] == '/':
			rpmlist = os.popen('rpm -qpi %s' % (url), 'r').read()
		else:
			# It should be an installed rpm
			rpmlist = os.popen('rpm -qi %s' % (url), 'r').read()
		return rpmlist

#************************************************************************************
# displayQuery(index,listbox)
#		- "index" is the index in the main listbox of selected rpm's
#		- opens a message window and displays the queried rpm(s)
#		- "file" is used when running xrpm by command line
#------------------------------------------------------------------------------------
	def displayQuery(self, index, listbox, file=None):
		
		self.dqindex = index		# have to make these global for tk, blahhhh!!
		self.dqindex_len = 0

		self.dqtextvar = StringVar()	# set these before we pack anything
		self.reload_message(listbox, 'N', file)

		dq = Toplevel(self)
		self.dq = dq
									
		dqmes = Frame(dq, bd=2, relief=SUNKEN, height='1000')
		dqmes.pack(side=TOP, padx=5, pady=7)

		dq.title('Query Package')
		dqtext = Message(dqmes, relief=RAISED,
						font='-adobe-courier-medium-r-normal--12-*-*-*-*-*-iso8859-1',
						width='1000')
		dqtext.pack(side=TOP)
		dqtext["textvariable"] = self.dqtextvar
		
		if len(self.dqindex) > 1:	# If we have multiple selections
			self.dqbut = Button(dq, text='Next Package',command=lambda x=self: x.reload_message(x.listbox, 'Y'), 
					activebackground='grey90')
			self.dqbut.pack(side=TOP, padx=4, expand=YES, fill=X)

		Button(dq, text='Close',
				command=dq.destroy, activebackground='grey90').pack(side=TOP, padx=4, expand=YES, fill=X)
		

	def reload_message(self,listbox, multi=None, file=None):
		if self.dqindex_len == len(self.dqindex):	# If we've looked at all the selecetions
			return									# return
		if multi == 'Y':
			if self.dqindex_len == len(self.dqindex)-1:
				Pack.forget(self.dqbut)				# loose this button for the last entry

		if not file:
			label = listbox.get(self.dqindex[self.dqindex_len])
		else:
			label = file
		
		if self.mode == INSTALLED or self.mode == GROUP:					# if newdir is ""None"" it's installed rpm
			s = label								# display it
		elif self.mode == FTP:						# if called from getftpsite()
			if self.con.uid:
				s = 'ftp://%s:%s@%s%s/%s' % (self.con.uid, self.con.passwd,self.con.connected_site, self.con.ftpdir, label)
			else:
				s = 'ftp://%s%s/%s' % (self.con.connected_site, self.con.ftpdir, label)
		else:									# it's a directory query so	
			s = os.path.join(self.newdir, label)	# join the root dir and the cwd

		if not file:
			self.mlbframe.config(cursor='watch')	# do this to give us a watch cursor
			self.master.update()					# while waiting.
			self.dqtextvar.set(self.query(s))              # return it here
			self.mlbframe.config(cursor='')		
			self.master.update()
	
			self.dqindex_len = self.dqindex_len + 1		# increment our selection counter
		else:
			self.dqtextvar.set(self.query(s))              # return it here
			self.master.update()
					
#************************************************************************************
# findDepend(url)
#------------------------------------------------------------------------------------
	def findDepend(self,url):
		if url[:6] == 'ftp://':
			os.system('rpm -qpR %s' % (url))
		elif url[:1] == '/':
			os.system('rpm -qpR %s' % (url))
		else:
			# It should be an installed rpm
			os.system('rpm -qR %s' % (url))


#************************************************************************************
# verify(url)
#------------------------------------------------------------------------------------
	def verify(self, url):
		# It should be an installed rpm
		list = os.popen('rpm -V %s' % (url), 'r').readlines()
		return list

#************************************************************************************
# displayVerify(label)
#		- "index" is the index in the main listbox of selected rpm's
#		- opens a lisbox and displays the verifyed files contained in the rpm(s)
#------------------------------------------------------------------------------------
	def displayVerify(self, index, listbox):
		self.vindex = index		# have to make these global for tk, blahhhh!!
		self.vindex_len = 0		# initialize counter to 0.  It counts up.

		v = Toplevel(self)
		v.title('Verify Package')

		self.vlabel = Label(v, text = ' ')
		self.vlabel.pack(side=TOP)

		self.vframe  = Frame(v, bd=3)
		self.vframe.pack(side=TOP, expand=YES, fill=BOTH, padx=5, pady=7)
		
		vscroll = Scrollbar(self.vframe, relief=SUNKEN)
		self.vlist   = Listbox(self.vframe, height=20, width=70,
				font='-adobe-courier-medium-r-normal--12-*-*-*-*-*-iso8859-1' )

		self.vlist.config(yscrollcommand=vscroll.set)
		self.vlist.pack(side=LEFT, expand=YES, fill=BOTH)

		vscroll.config(command=self.vlist.yview, relief=SUNKEN)
		vscroll.pack(side=RIGHT, fill=BOTH)

		if len(self.vindex) > 1:	# If we have multiple selections
			self.vbut = Button(v, text='Next Package',command=lambda x=self: x.reload_verifybox(x.listbox, 'Y'), 
					activebackground='grey90')
			self.vbut.pack(side=TOP, padx=4, expand=YES, fill=X)

		Button(v, text='Close', command=v.destroy, activebackground='grey90').pack(side=TOP, padx=4, expand=YES, fill=X)
		
		self.reload_verifybox(listbox)

	def reload_verifybox(self,listbox, multi=None):
		if self.vindex_len == len(self.vindex):	# If we've looked at all the selecetions
			return									# return
		if multi == 'Y':
			if self.vindex_len == len(self.vindex)-1:
				Pack.forget(self.vbut)				# loose this button for the last entry

		label = listbox.get(self.vindex[self.vindex_len])

		self.vlabel.config(text=label)
				
		self.vframe.config(cursor='watch')	# do this to give us a watch cursor
		self.master.update()					# while waiting.
		nn = self.verify(label)
		self.vframe.config(cursor='')		
		self.master.update()
		self.vindex_len = self.vindex_len + 1		# increment our selection counter

		pos = 0
		for label in nn:    # add to list-box
			self.vlist.insert(pos, label)                      # and menu/toolbars
			pos = pos + 1
			
		
#************************************************************************************
# remove the rpm
# remove(index, listbox)
#		- "index" is the rpm(s) to remove, indexed from the "listbox"
#
# from the rpm manual 
#   rpm -e <package_name>+
#   --noscripts - Don't execute the preuninstall or postuninstall scripts.
#	--nodeps    - Don't check dependencies before uninstalling the packages.
#	--test      - Don't really uninstall anything, just go throughthe motions. -vv option.
#------------------------------------------------------------------------------------
	def remove(self, index, listbox):
		
		# initialize vars
		self.rmlist = []		# have to make these global for tk, blahhhh!!

		for i in index:			# build a list of rpm's to remove
			self.rmlist.append(listbox.get(i))		

		self.noscripts = IntVar()	# We have to set these up for the checkbuttons
		self.nodeps = IntVar()		# to get the value of the checkbutton set the variable
		self.test = IntVar()		# like this - self.rbscript["variable"] = self.noscripts
		self.noscripts.set(0)		# then get the value with self.noscripts.get()
		self.nodeps.set(0)
		self.test.set(0)
		
		self.rmwid = Toplevel(self)
		self.rmwid.title('Remove RPM')

		Label(self.rmwid,text=" ").pack(side=TOP)
		label= Label(self.rmwid, 
				text="You've selected the following packages to remove",
				font='-adobe-helvetica-bold-r-normal--12-*-*-*-*-*-iso8859-1')
		label.pack(side=TOP)

		rmframe  = Frame(self.rmwid, bd=5)							   # frame for scrolled text
		rmframe.pack(side=TOP, expand=YES, fill=BOTH)
		
		self.rmtext = ScrolledText(rmframe, height=15, width=40)
		self.rmtext.pack(side=LEFT, expand=YES, fill=Y)

		toolbar= Frame(rmframe, cursor='hand2', relief=SUNKEN, bd=2)
		toolbar.pack(side=LEFT, fill=Y)

		optionsButton = Button(toolbar, text='Options', command=self.rmOptions, width=10, activebackground='grey90')
		optionsButton.pack(side=TOP)
		removeButton = Button(toolbar, text='Remove', command=self.rmAction, width=10, 
				activebackground='red')
		removeButton.pack(side=TOP)
		Button(toolbar, text='Close', command=self.rmwid.destroy, width=10, activebackground='grey90').pack(side=BOTTOM)

		for i in index:
			label = listbox.get(i)+'\n'
			self.rmtext.insert('end', label)


#************************************************************************************
# actually perform the removal 
#------------------------------------------------------------------------------------						
	def rmAction(self):
		rmOptions = ''					# initialize to null value
		if self.noscripts.get() == 1:
			rmOptions = '--noscripts'
		if self.nodeps.get() == 1:
			rmOptions = rmOptions+' --nodeps'
		if self.test.get() == 1:
			rmOptions = rmOptions+' --test'

		# if your not root you can't do this unless your just testing
		if os.getuid() != 0 and self.test.get() == 0:
			attention('You must su or login as root to remove installed packages from this system.\n\n')
			return
		else:
			self.master.update()
			result = []
			for i in self.rmlist:
				d = alert('\n\n Removing '+i+'\n\n')
				self.master.update()
				stdio, stderr = self.runCommand('rpm -e '+rmOptions+' '+i)
				if stderr:
					d1 = Dialog(None, title = 'Attention!',
							text = stderr,
							bitmap =  DIALOG_ICON,
							default = 0,
							strings = ('Continue','Cancel Remaining'))
					if d1.num == 1:
						d.destroy()
						self.master.update()
						break
				d.destroy()
				# ------------------ usage logging --------------------------------
				# file format is  as follows:
				# "command:package:install options:host:path:date"
				# command can be either "Removed" or "Installed"
				# host and path can be empty as is the case when removing.
				# date is output from unix 'date ' command.
				# Ex: removed:blat-0.2.1-1:::(date)
				#------------------------------------------------------------------
				if self.ConfigUseLog.get() and self.test.get() == 0:		# if we're using logging do it now
					try:
						fd = open('/var/lib/rpm/xrpm-use.log', 'a') 	# open in append/create mode
					except:
						attention("Can't open /var/lib/rpm/xrpm-use.log file.  You must be root!")
						self.rmwid.destroy()
						self.getinstalled()		# rescan the installed rpm's
						return
					date = os.popen('date +"%y/%m/%d at %k%M"', 'r').read()
					fd.write('Removed:'+i+'::::'+date)
					fd.close()

				# check above lines
							
			self.rmwid.destroy()
			self.getinstalled()		# rescan the installed rpm's

	def rmOptions(self):
		wid = Toplevel(self)
		wid.title('Remove Options')

		label= Label(wid, text=' ')								# give us a little room at the top
		label.pack(side=TOP)

		rmtoolbar= Frame(wid, cursor='hand2', relief=SUNKEN, bd=2)    # frame for cb's and rm button
		rmtoolbar.pack(side=TOP, expand=YES, fill=BOTH, padx=5)
		
		rbframe = Frame(rmtoolbar, bd=1, relief=RAISED)	   # frame for checkbuttons
		rbframe.pack(side=TOP, expand=YES, fill=BOTH)

# checkbutton stuff
		rblabel= Label(rbframe, text='Remove Options ',font='-adobe-helvetica-bold-r-normal--12-*-*-*-*-*-iso8859-1')
		rblabel.pack(side=TOP)
		rbscript= Checkbutton(rbframe, 
		   text='[NOSCRIPTS] Don''t execute the preuninstall or postuninstall scripts', 
		   anchor=W)
		rbscript.pack(side=TOP, expand=YES, fill=X)
		rbdeps= Checkbutton(rbframe, 
		   text='[NODEPS] Don''t check dependencies before uninstalling the packages.', 
		   anchor=W)
		rbdeps.pack(side=TOP, expand=YES, fill=X)
		rbtest= Checkbutton(rbframe, 
		   text='[TEST] Don''t uninstall anything, just go through the motions.', 
		   anchor=W)
		rbtest.pack(side=TOP, expand=YES, fill=X)
		rblabel1= Label(rbframe, text=' ')
		rblabel1.pack(side=TOP)
		
		rbscript["variable"] = self.noscripts  # watch these vars
		rbdeps["variable"] = self.nodeps
		rbtest["variable"] = self.test
# end checkbutton stuff
		
		rmlabel = Label(wid, text=' ')
		rmlabel.pack(side=TOP)

		Button(wid, text='Close', 
				command=wid.destroy, width=10, activebackground='grey90').pack(side=TOP, padx=4, expand=YES, fill=X)


#************************************************************************************
# install the rpm
# install(package, listbox, file))
#		- "package" is the rpm to install
#		- "listbox" is address of listbox object
#		- "file" is file as called from the command line
#
# from the rpm manual 
#  rpm -i [install-options] <package_file>+
#
#  The  <package_file>  may be specified as an ftp style URL,
#  in which case the package will be downloaded before  being
#  installed.  See FTP OPTIONS for information on RPM's built
#  in ftp support.
#
#  --force        Same as using --replacepkgs, --replacefiles, and --oldpackage.
#  -h, --hash     Print 50 hash marks as the package archive is unpacked.
#  --oldpackage   Allow an upgrade to replace a newer package with an older one.
#  --percent      Print percentages as files are unpacked from the package archive.
#  --replacefiles Install the packages even if they replace files from other, already installed, packages.
#  --replacepkgs  Install the packages even if some of them are already installed on this system.
#  --nodeps       Don't do a dependency check before installing the new package.
#  --noscripts    Don't execute the preinstall or postinstall scripts.
#  --excludedocs  Don't install any files which are marked as documentation.
#  --test         Do not install the package, simply check for and report potential conflicts.
#  -U, --upgrade  Upgrade the package currently installed to the version in the new RPM.
#------------------------------------------------------------------------------------
	def install(self, index, listbox, file=None):
		# initialize vars
		self.inlist = []		# have to make these global for tk, blahhhh!!

		if file:
			self.inlist.append(file)
		else:		
			for i in index:			# build a list of rpm's to install
				self.inlist.append(listbox.get(i))		
			
		self.force = IntVar()			# We have to set these up for the checkbuttons
		self.oldpackage = IntVar()		# to get the value of the checkbutton set the variable
		self.replacefiles = IntVar()	# like this - self.rbscript["variable"] = self.noscripts
		self.replacepkgs = IntVar()		# then get the value with self.noscripts.get()
		self.noscripts = IntVar()	
		self.nodeps = IntVar()		
		self.excludedocs = IntVar()
		self.test = IntVar()		
		self.upgrade = IntVar()		
#		self.verbose = IntVar()
		self.deleteDl = IntVar()

		self.force.set(0)
		self.oldpackage.set(0)
		self.replacefiles.set(0)
		self.replacepkgs.set(0)
		self.noscripts.set(0)
		self.nodeps.set(0)
		self.excludedocs.set(0)
		self.test.set(0)
		self.upgrade.set(0)
#		self.verbose.set(0)
		self.deleteDl.set(0)

		self.inwid = Toplevel(self)
		self.inwid.title('Install RPM')

		Label(self.inwid,text=" ").pack(side=TOP)
		label= Label(self.inwid, 
				text="You've selected the following packages to install",
				font='-adobe-helvetica-bold-r-normal--12-*-*-*-*-*-iso8859-1')
		label.pack(side=TOP)

		inframe  = Frame(self.inwid, bd=5)			   # frame for scrolled text
		inframe.pack(side=TOP, expand=YES, fill=BOTH)
		
		self.intext = ScrolledText(inframe, height=15, width=40)
		self.intext.pack(side=LEFT, expand=YES, fill=BOTH)

		toolbar= Frame(inframe, cursor='hand2', relief=SUNKEN, bd=2)
		toolbar.pack(side=LEFT, fill=Y)
		optionsButton = Button(toolbar, text='Options', command=self.inOptions, width=10, activebackground='grey90')
		optionsButton.pack(side=TOP)
		installButton = Button(toolbar, text='Install', command=self.inAction, width=10, 
				activebackground='forestgreen')
		installButton.pack(side=TOP)
		Button(toolbar, text='Close', command=self.inwid.destroy, width=10, activebackground='grey90').pack(side=BOTTOM)

		if file:
			self.intext.insert('end', file+'\n')
		else:
			for i in index:
				label = listbox.get(i)
				self.intext.insert('end', label+'\n')
			
#--------------------------------------------------------------------------------------
	def inOptions(self):
		wid = Toplevel(self)
		wid.title('Install Options')

		label= Label(wid, text=' ')								# give us a little room at the top
		label.pack(side=TOP)

		intoolbar= Frame(wid, cursor='hand2', relief=SUNKEN, bd=2)    # frame for cb's and rm button
		intoolbar.pack(side=TOP, expand=YES, fill=BOTH, padx=5)
		
		rbframe = Frame(intoolbar, bd=1, relief=RAISED)	   # frame for checkbuttons
		rbframe.pack(side=TOP, expand=YES, fill=X)

		cbframe = Frame(intoolbar, bd=1, relief=RAISED)		# frame for checkbuttons
		cbframe.pack(side=TOP, expand=YES, fill=X)

# checkbutton stuff
		cblabel= Label(cbframe, text='Install Options ', font='-adobe-helvetica-bold-r-normal--12-*-*-*-*-*-iso8859-1')
		cblabel.pack(side=TOP)

		cbforce= Checkbutton(cbframe, 
		   text='[FORCE] Same as using both --replacepkgs, --replacefiles, and --oldpackage.', 
		   anchor=W, command=self.forceCheck)
		cbforce.pack(side=TOP, expand=YES, fill=X)
		cboldpackage= Checkbutton(cbframe, 
		   text='[OLDPACKAGE] Allow an upgrade to replace a newer package with an older one.', 
		   anchor=W, command=self.oldpackageCheck)
		cboldpackage.pack(side=TOP, expand=YES, fill=X)
		cbreplacefiles= Checkbutton(cbframe, 
		   text='[REPLACEFILES] Install the packages even if they replace files from other packages', 
		   anchor=W, command=self.replacefilesCheck)
		cbreplacefiles.pack(side=TOP, expand=YES, fill=X)
		cbreplacepkgs= Checkbutton(cbframe, 
		   text='[REPLACEPKGS] Install the packages even if some of them are already installed.', 
		   anchor=W, command=self.replacepkgsCheck)
		cbreplacepkgs.pack(side=TOP, expand=YES, fill=X)
		cbnoscripts= Checkbutton(cbframe, 
		   text='[NOSCRIPTS] Don''t execute the preinstall or postinstall scripts', 
		   anchor=W)
		cbnoscripts.pack(side=TOP, expand=YES, fill=X)
		cbnodeps= Checkbutton(cbframe, 
		   text='[NODEPS] Don''t check dependencies before installing the package.', 
		   anchor=W)
		cbnodeps.pack(side=TOP, expand=YES, fill=X)
		cbexcludedocs= Checkbutton(cbframe, 
		   text='[EXCLUDEDOCS] Don''t install any files which are marked as documentation.', 
		   anchor=W)
		cbexcludedocs.pack(side=TOP, expand=YES, fill=X)
		cbupgrade= Checkbutton(cbframe, 
		   text='[UPGRADE] Upgrade the package currently installed to the version in the new RPM.', 
		   anchor=W)
		cbupgrade.pack(side=TOP, expand=YES, fill=X)
#		cbverbose= Checkbutton(cbframe, 
#		   text='[VERBOSE] Print lots of information about installation.', 
#		   anchor=W)
#		cbverbose.pack(side=TOP, expand=YES, fill=X)
		cbtest= Checkbutton(cbframe, 
		   text='[TEST] Don''t install anything, just go through the motions.', 
		   anchor=W)
		cbtest.pack(side=TOP, expand=YES, fill=X)
		if self.mode == FTP:
			cbdeleteDl= Checkbutton(cbframe, 
			   text="[DELETE] Delete ftp'd rpm after download.", 
			   anchor=W)
			cbdeleteDl.pack(side=TOP, expand=YES, fill=X)

		cblabel1= Label(cbframe, text=' ')
		cblabel1.pack(side=TOP)

		cbforce["variable"] = self.force
		cboldpackage["variable"] = self.oldpackage
		cbreplacefiles["variable"] = self.replacefiles
		cbreplacepkgs["variable"] = self.replacepkgs
		cbnoscripts["variable"] = self.noscripts  # watch these vars
		cbnodeps["variable"] = self.nodeps
		cbexcludedocs["variable"] = self.excludedocs
		cbtest["variable"] = self.test
		cbupgrade["variable"] = self.upgrade
#		cbverbose["variable"] = self.verbose
		if self.mode == FTP:
			cbdeleteDl["variable"] = self.deleteDl
		
# end checkbutton stuff
		inlabel = Label(wid, text=' ')
		inlabel.pack(side=TOP)

		Button(wid, text='Close', 
				command=wid.destroy, width=10, activebackground='grey90').pack(side=TOP, padx=4, expand=YES, fill=X)
		
#************************************************************************************
# these functions make sure that if one option is set, the other options that 
# would be redundant are not set
# forceCheck()
#		- right now this just covers the --force option and dependents
#------------------------------------------------------------------------------------ 
	def forceCheck(self):
		if self.force.get() == 1:
			self.replacefiles.set(0)
			self.replacepkgs.set(0)
			self.oldpackage.set(0)
	def oldpackageCheck(self):
		if self.oldpackage.get() == 1:
			self.force.set(0)
	def replacefilesCheck(self):
		if self.replacefiles.get() == 1:
			self.force.set(0)
	def replacepkgsCheck(self):
		if self.replacepkgs.get() == 1:
			self.force.set(0)

#************************************************************************************
# actually perform the installation 
# inAction()
#		- get all the installation options
#		- runs the rpm command and returns the results in a textbox
#------------------------------------------------------------------------------------						
	def inAction(self):
		inOptions = ''					# initialize to null value
		if self.force.get() == 1:
			inOptions = inOptions+'--force'
		else:
			if self.oldpackage.get() == 1:
				inOptions = inOptions+' --oldpackage'
			if self.replacefiles.get() == 1:
				inOptions = inOptions+' --replacefiles'
			if self.replacepkgs.get() == 1:
				inOptions = inOptions+'--replacepkgs'

		if self.noscripts.get() == 1:
			inOptions = inOptions+' --noscripts'
		if self.nodeps.get() == 1:
			inOptions = inOptions+' --nodeps'

		if self.excludedocs.get() == 1:
			inOptions = inOptions+'--excludedocs'
		if self.upgrade.get() == 1:
			inOptions = inOptions+' --upgrade'
		if self.test.get() == 1:
			inOptions = inOptions+' --test'
		
		# if your not root you can't do this unless your just testing
		if os.getuid() != 0 and self.test.get() == 0:
			attention('You must su or login as root to install packages from this system.\n\n')
			return
		else:
			result = []
			for i in self.inlist:
				d = alert('\n\n Installing '+i+'\n\n')
				self.master.update()
				stderr = self.installPackage(i,inOptions)	# individual package install
				d.destroy()
				self.master.update()
				
			self.inwid.destroy()	# after we're finished get rid of all the windows

#----------------------------------------------------------------------------------
	def installPackage(self, package, options):

		if self.mode == FTP:		# if called from getftpsite() make a temp filename
			s = '/tmp/%s' % (package)	# we put it in the /tmp directory
		elif self.mode == COMMAND:	# we're called from the command line
			s = package
		else:
			if self.newdir[-2:] == '/.':		# For some reason I have a /. at the end of the
				self.newdir = self.newdir[:-2]	# newdir???, anyway get rid of it.
			s = os.path.join(self.newdir, package)	# join the dir and filename
		
		s1 = package[:rfind(package[:-5], '.')]	# chop off ".*.rpm"

		continueInstall = None		# flag to continue if we're testing etc.

		alreadyInstalled = None		# set this up

		# if it's already installed there's no point unless we're testing.
		if os.popen('rpm -qa | grep %s' % (s1), 'r').read():
			attention('\n\n'+package+' is already installed.\n\n')
			alreadyInstalled = 'yes'	# cancel depend check if installed
			
			if self.force.get() == 1 or self.oldpackage.get() == 1 or self.replacepkgs.get() == 1:
				d = Dialog(None, title = 'Attention!',
					text = 'You have requested this package installation be forced.'
							' It will replace the package that is already installed.\n\n'
							'Do you want to continue?',
					bitmap =  DIALOG_ICON,
					default = 0,
					strings = ('Cancel','Continue'))

				if d.num == 1:
					continueInstall = 'yes'
								
			if self.test.get() == 1:
				continueInstall = 'yes'
		
		else:	# we're here if it's not already installed
			continueInstall = 'yes'
		
		if self.mode == FTP and continueInstall == 'yes':
		# get the package since we have to do that any way, store it in /tmp
			self.con.ftpget(self.con.ftpdir, package)

		# check for any dependencies required
		if self.nodeps.get() == 0 and continueInstall == 'yes' and alreadyInstalled != 'yes' and self.upgrade.get() == 0:
			junk, dependCheck = self.runCommand('rpm -i --test '+s)
		else:
			dependCheck = None

		if dependCheck:
			attention(package+'\nFailed dependancy check:\n'+dependCheck)
			continueInstall = None
			
			if self.mode == FTP and self.deleteDl.get() == 1:
				d1 = Dialog(None, title = 'Attention!',
					text = 'This package has failed the dependancy check, and is marked for deletion.'
							' You may choose to SAVE the file and it will NOT have to be downloaded'
							' again.  If you don''t delete the package you can install it from the /tmp'
							' directory at any time.',
					bitmap =  DIALOG_ICON,
					default = 0,
					strings = ('Save','Delete'))

				if d1.num == 0:			# save button pressed turn delete off
					self.deleteDl.set(0)
					
			if self.test.get() == 1:	# test mode?
				continueInstall = 'yes'

		stderr = None	# set default return value
		if continueInstall == 'yes':
			stdio, stderr = self.runCommand('rpm -i '+options+' '+s)

			# ------------------ usage logging --------------------------------
			# file format is  as follows:
			# "command:package:install options:host:path:date"
			# command can be either "Removed" or "Installed"
			# host and path can be empty as is the case when removing.
			# date is output from unix 'date ' command.
			# Ex: removed:blat-0.2.1-1:::(date)
			#------------------------------------------------------------------
			if self.ConfigUseLog.get() and self.test.get() == 0:	# if we're using logging do it now
				try:
					fd = open('/var/lib/rpm/xrpm-use.log', 'a') 	# open in append/create mode
				except:
					attention("Can't open /var/lib/rpm/xrpm-use.log file.")
					self.inwid.destroy()	# after we're finished get rid of all the windows
					return
					
				if self.mode == FTP:
					host = self.con.connected_site
					path = self.con.ftpdir
				elif self.mode == DIRECTORY:
					host = ''
					path = self.newdir
				elif self.mode == COMMAND:
					host = ''
					path, package = os.path.split(s)										
				else:
					host = ''
					path = ''
					
				date = os.popen('date +"%y/%m/%d at %k%M"', 'r').read()
				fd.write('Installed:'+package+':'+options+':'+host+':'+path+':'+date)
				fd.close()
	    	
		if self.mode == FTP:
			if self.deleteDl.get() == 1:
				os.unlink(s)
		return stderr

#************************************************************************************
# I did this because on my Linux system the darn python (1.4) popen command won't
# redirect stderr with 2>&1 ?????
# runCommand(command)
#		- "command" is the shell command ro execute
#		- returns "io" aka stdio and "err" aka stderr
#------------------------------------------------------------------------------------
	def runCommand(self, command):
		if command[-1:] == '\n':
			command = command[0:-1]
		runme = command+' >/tmp/xrpm.stdio 2>/tmp/xrpm.stderr'
		os.system(runme)

		fd = open('/tmp/xrpm.stdio', 'r')
		io = fd.read()
		fd.close()
				
		fd = open('/tmp/xrpm.stderr', 'r')
		err = fd.read()
		fd.close()
		
		os.unlink('/tmp/xrpm.stdio')
		os.unlink('/tmp/xrpm.stderr')
		
		return io, err

		
# **************************** Main **********************************
def test():

	ftpfd = open('/tmp/'+'index', 'w')
	ftplist.ftpget(site, '/pub', 'index')
	ftpfd.close()

		
if __name__ == '__main__': test()
	
		
		
		
