#!/usr/bin/python

""" 
	xrpm - version 2.1
	
    Copyright (C) 1997  George A. Farris
        george@gmsys.com - www.gmsys.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""
# Modes that xrpm can be in.
INSTALLED = 'ins'
DIRECTORY = 'dir'
GROUP     = 'grp'
FTP       = 'ftp'
COMMAND   = 'com'	# called from command line

import os
import sys

from Tkinter import *
from rpm import Rpm
from string import split, find
from gmsgui import attention, listfile
from gui import GetDir, GetFtp
from ScrolledText import *

class Xrpm(Frame, Rpm):

	def __init__(self, parent=None):
		Frame.__init__(self, parent)
		self.pack(expand=YES, fill=BOTH)
		
		self.mode = INSTALLED	
		self.xrpmdir = os.environ['PYTHONPATH']

		# Config file options. ----------------------------------
		self.ConfigUseLog = IntVar()		# Options menu "Enable Log"
		self.ConfigUseLog.set(0)
		self.ConfigUseTkman = IntVar()	# Options menu "Use TkMan"
		self.ConfigUseTkman.set(0)
		
		try:
			fd = open('/etc/xrpm.conf', 'r')
		except:
			attention('Your "/etc/xrpm.conf" file is missing or has the wrong permissions!')
			sys.exit()
		
		try:
			self.ConfigUseLog.set(split(fd.readline())[1])
			self.ConfigUseTkman.set(split(fd.readline())[1])
		except:
			attention('There is a problem reading your /etc/xrpm.conf file, using default options')
		# ------------ end of config stuff -----------------------
		
		if len(sys.argv) > 1:

			if sys.argv[1] == '-q' or sys.argv[1] == '-l' or sys.argv[1] == '-i':
				index = []						# fake listbox index
				index.append(0)					# point it to item 0
				self.master.withdraw()			# hide top frame, we don't need it (sort of)
				s = sys.argv[2][0:1]			# get first char of filename
				if s != '/' and s != '~':		# add a path if it isn't there
					argv2 = os.getcwd()+'/'+sys.argv[2]
				else:
					argv2 = sys.argv[2]
					
			# Query from command line
			if sys.argv[1] == '-q':
				self.displayQuery(index,None,argv2)
				self.wait_window(self.dq)
			# List files from command line
			if sys.argv[1] == '-l':
				self.displayFiles(index,None,argv2)
				self.wait_window(self.flb)
			# Install from command line
			if sys.argv[1] == '-i':
				self.mode = COMMAND
				self.install(index, None, argv2)
				self.wait_window(self.inwid)

			if sys.argv[1] != '-q' and sys.argv[1] != '-l' and sys.argv[1] != '-i':
				print 'Usage: xrpm [OPTION] RPM\n'
				print '  -i	Install the package.'
				print '  -l	List package files.'
				print '  -q	Query the package.\n'
				print 'Example: xrpm -q rxvt-2.4.1-1.i386.rpm\n'
				
			sys.exit()

			
		self.createWidgets()
		self.master.title("XRPM-2.1")
		
		self.master.group('.')
		self.master.command(sys.argv)
		self.master.iconname("xrpm")
		self.newdir = None

				
	def createWidgets(self):
		self.makeMenuBar()

		self.makeListBox()
		self.makeToolBar()
		self.label = Label(text='Installed rpms',font='-adobe-helvetica-medium-r-normal--12-*-*-*-*-*-iso8859-1')
		self.label.pack(side=BOTTOM)
		self.labelpath = Label(text='',font='-adobe-helvetica-medium-r-normal--12-*-*-*-*-*-iso8859-1')
		self.labelpath.pack(side=BOTTOM)

#************************************************************************************
# called from createWidgets()
# makeListBox()
# 		- makes the main listbox with scrollbar
#		- loads a list of rpm's currently installed on the system
#		- binds the <Double-1> mouse click to handleList()
#------------------------------------------------------------------------------------
	def makeListBox(self):                     # add listbox in middle
		self.mlbframe  = Frame(self, bd=3)
		self.mlbframe.pack(side=LEFT, expand=YES, fill=BOTH)

		f1 = Frame(self.mlbframe)
		f1.pack(side=LEFT, expand=YES, fill=BOTH)

		self.groupLabel = Label(f1, text='Package Name', relief=FLAT, anchor=W)
		self.groupLabel.pack(side=TOP, fill=X)

		scroll = Scrollbar(self.mlbframe)		# config now so list knows about it
		list   = Listbox(f1, height=20, width=40, font='-adobe-courier-medium-r-normal--12-*-*-*-*-*-iso8859-1' )
		list.config(yscrollcommand=scroll.set)
		list.pack(side=LEFT, expand=YES, fill=BOTH)

		group =  Listbox(f1, height=20, width=20, font='-adobe-courier-medium-r-normal--12-*-*-*-*-*-iso8859-1' )
		group.config(yscrollcommand=scroll.set)
		group.pack(side=RIGHT, expand=YES, fill=BOTH)

		scroll.config(command=list.yview, relief=SUNKEN)
		scroll.pack(side=RIGHT, fill=Y)

		pos = 0
		rpmlist = os.popen('rpm -qa | sort', 'r').readlines()
		for label in rpmlist:    # add to list-box
			list.insert(pos, label[:-1])   # don't print the newlines
			pos = pos + 1

		list.config(selectmode=EXTENDED, setgrid=1)        # select,resize modes
		list.bind('<Double-1>', self.handleList)         # set event handler
		
		self.listbox = list   
		self.groupbox = group
				
		# Set up the listboxes for dual scroll with one scrollbar
		self.listbox.config(yscrollcommand=(scroll, 'set'))
		self.groupbox.config(yscrollcommand=(scroll, 'set'))
		scroll.config(command=(self.bindyview))

		Pack.forget(self.groupbox)	# Hide the dir scroll bar for now

# This is the double listbox scroll command
	def bindyview(self, a=None, b=None, c=None):
		self.listbox.yview(a, b, c)
		self.groupbox.yview(a, b, c)

	def makeToolBar(self):
		toolbar= Frame(self, cursor='hand2', relief=SUNKEN, bd=2)
		toolbar.pack(side=RIGHT, anchor=E, fill=Y, padx=2, pady=2)

		qbm = PhotoImage(file=self.xrpmdir+'/info.gif')
		fbm = PhotoImage(file=self.xrpmdir+'/file.gif')
		ibm = PhotoImage(file=self.xrpmdir+'/install.gif')
		rbm = PhotoImage(file=self.xrpmdir+'/remove.gif')
		ebm = PhotoImage(file=self.xrpmdir+'/quit.gif')  	 

		self.queryButton = Button(toolbar, image=qbm, command=self.handleQuery,width=10, activebackground='grey90')
		self.queryButton.pack(side=TOP, fill=X)
		self.listButton = Button(toolbar, image=fbm, command=self.handleFiles, width=10, activebackground='grey90')
		self.listButton.pack(side=TOP, fill=X)
#		self.verifyButton = Button(toolbar, text='Verify', command=self.handleVerify, width=10, activebackground='grey90')
#		self.verifyButton.pack(side=TOP, fill=X)
		self.removeButton = Button(toolbar, image=rbm, command=self.removerpm, width=10, activebackground='grey90')
		self.removeButton.pack(side=TOP, fill=X)
		self.installButton = Button(toolbar, image=ibm, command=self.installrpm, width=10, activebackground='grey90', state='disabled')
		self.installButton.pack(side=TOP, fill=X)
		Button(toolbar, image=ebm, command=self.quit, width=74, activebackground='grey90').pack(side=BOTTOM, fill=X)
	
	def makeMenuBar(self):
		self.menubar = Frame(self, relief=RAISED, bd=2)
		self.menubar.pack(side=TOP, fill=X)
		self.fileMenu()
		self.groupMenu()
		self.optionMenu()
		self.helpMenu()
		
	def fileMenu(self):
		mbutton = Menubutton(self.menubar, text='File', underline=0)
		mbutton.pack(side=LEFT)
		menu = Menu(mbutton)
		menu.add_command(label='Open Directory', command=self.getdir)
		menu.add_command(label='Open FTP Site', command=self.getftpsite)
		menu.add_command(label='List Installed', command=self.getinstalled)
		menu.add_command(label='View Log File', command=self.getlogfile)
		menu.add_command(label='Exit...', command=self.quit)
		mbutton['menu'] = menu
		return mbutton

	def groupMenu(self):
		mainGroups = self.getMainGroups()	# search all the packages fro high level groups
		mbutton = Menubutton(self.menubar, text='Groups', underline=0)
		mbutton.pack(side=LEFT)
		self.gmenu = Menu(mbutton)
		for menugroup in mainGroups:
			self.gmenu.add_command(label=menugroup, command=lambda item=menugroup, f=self.getgroup: f(item))
		mbutton['menu'] = self.gmenu
		return mbutton

	def optionMenu(self):
		mbutton = Menubutton(self.menubar, text='Options', underline=0)
		mbutton.pack(side=LEFT)
		menu = Menu(mbutton)
		menu.add_checkbutton(label=' Enable Log ', variable=self.ConfigUseLog)
		menu.add_checkbutton(label=' Use TkMan  ', variable=self.ConfigUseTkman)
		menu.add_command(label=' Save Options ', command=self.saveOptions)
		mbutton['menu'] = menu
		return mbutton

	def helpMenu(self):
		mbutton = Menubutton(self.menubar, text='Help', underline=0)
		mbutton.pack(side=RIGHT)
		menu = Menu(mbutton)
		menu.add_command(label=' About  ', command=self.about)
		menu.add_command(label=' Manual  ', command=self.help)
		menu.add_command(label=' License ', command=self.license)
		mbutton['menu'] = menu
		return mbutton

	def getMainGroups(self):
		glist = []
		i = 0
		s = os.popen('rpm  -qa  --queryformat "%{GROUP}\n"').readlines()
		for s1 in s:
			maingroup = split(s1,'/')[0]	# get the first group in the path
			if maingroup[-1:] == '\n':		# remove any newlines
				glist.append(maingroup[0:-1])
			else:
				glist.append(maingroup)
		glist.sort()
		mlist = []

		last = ''				# lets remove all the duplicates
		for current in glist:
			if current != last:
				mlist.append(current)
				last = current

		return mlist
		
#************************************************************************************
# saveOption()
#		- Writes new options to the /etc/xrpm.conf file.
#------------------------------------------------------------------------------------
	def saveOptions(self):
		
		try:
			fdo = open('/etc/xrpm.conf', 'w')
		except:
			attention("Can't open /etc/xrpm.conf for writing.  You should be root to change these.")
			return
		
		ul = str(self.ConfigUseLog.get())		# convert to strings
		ut = str(self.ConfigUseTkman.get())
		
		fdo.write('UseLog '+ul+'\n')
		fdo.write('UseTkman '+ut+'\n')
		fdo.close()
	
	
#************************************************************************************
# about()
#		- opens a message window and displays the blerb about xrpm
#------------------------------------------------------------------------------------
	def about(self):

		title='\n  Xrpm  - Version 2.1  \n\n'
		copyright='  Copyright (C) 1997, 1998  George A. Farris  \n\n'
		email='  george@gmsys.com  \n\n'
		web='  http://www.gmsys.com  \n'
		
		ab = Toplevel(self)
		ab.mes = Frame(ab, bd=2, relief=SUNKEN)
		ab.mes.pack(side=TOP, padx=5, pady=7)
		
		ab.title('About XRPM')
		Message(ab.mes, relief=RAISED,
				font='-adobe-helvetica-bold-r-normal--14-*-*-*-*-*-iso8859-1',
				text=title+copyright+email+web,
				width='1000').pack(side=TOP)
				
		Button(ab, text='Close', 
				command=ab.destroy, activebackground='grey90').pack(side=TOP, padx=4, fill=X)

#************************************************************************************
# help()
#		- opens a message window and displays the xrpm manual
#------------------------------------------------------------------------------------
	def help(self):
		wid = Toplevel(self)
		wid.title('XRPM Help')

		label= Label(wid, text='XRPM Manual')
		label.pack(side=TOP)

		hframe  = Frame(wid, bd=5)
		hframe.pack(side=TOP, expand=YES, fill=BOTH)

		
		self.htext = ScrolledText(hframe, height=25, width=78)
		self.htext.pack(side=TOP, expand=YES, fill=BOTH)

		Button(wid, text='Close', 
				command=wid.destroy, width=10, activebackground='grey90').pack(side=BOTTOM, padx=4, fill=X)


		fd = open(self.xrpmdir+'/manual', 'r')

		for p in fd.readlines():    # add to text-box
			self.htext.insert('end', p)

#************************************************************************************
# license()
#		- opens a message window and displays the xrpm license
#------------------------------------------------------------------------------------
	def license(self):
		wid = Toplevel(self)
		wid.title('XRPM License')

		label= Label(wid, text='XRPM License')
		label.pack(side=TOP)

		lframe  = Frame(wid, bd=5)
		lframe.pack(side=TOP, expand=YES, fill=BOTH)
		
		self.ltext = ScrolledText(lframe, height=25, width=78)
		self.ltext.pack(side=TOP, expand=YES, fill=BOTH)

		Button(wid, text='Close', 
				command=wid.destroy, width=10, activebackground='grey90').pack(side=BOTTOM, padx=4, expand=YES, fill=X)

		fd = open(self.xrpmdir+'/license', 'r')

		for p in fd.readlines():    # add to text-box
			self.ltext.insert('end', p)


#************************************************************************************
# go here from double click in main window
# handleList(event)
#		- "event" is a window event (double click), we don't really see it
#		- we do a query of the selected rpm on a double click
#------------------------------------------------------------------------------------
	def handleList(self, event):
		index = self.listbox.curselection()     # on listbox double-click
		self.displayQuery(index, self.listbox)
#************************************************************************************
# go here from the "Query" button in the right button bar
# handleQuery()
#		- we do a query of the selected rpm when this button is pushed
#------------------------------------------------------------------------------------
	def handleQuery(self):
		index = self.listbox.curselection()     # fetch selection text	
		if index:		# if no selection don't do anything.
			self.displayQuery(index, self.listbox)

#************************************************************************************
# go here from "List Files" button in right button bar
# handleFiles()
#		- we do a list of the files in the selected rpm
#------------------------------------------------------------------------------------
	def handleFiles(self):
		index = self.listbox.curselection()     # fetch selection text
		if index:
			self.displayFiles(index, self.listbox)  

#************************************************************************************
# go here from "Verify" button in right button bar
# handleVerify()
#		- we do a verify of the files in the selected rpm
#------------------------------------------------------------------------------------
	def handleVerify(self):
		if self.mode != INSTALLED:
			return
		index = self.listbox.curselection()     # fetch selection text
		if index:
			self.displayVerify(index, self.listbox)  

#************************************************************************************
# go here from "Install" button
# install()
#
#------------------------------------------------------------------------------------
	def installrpm(self):
		index = self.listbox.curselection()     # fetch selection text
		if index:
			self.install(index,self.listbox)
		
#************************************************************************************
# go here from "Remove" button
# remove()
#
#------------------------------------------------------------------------------------
	def removerpm(self):
		index = self.listbox.curselection()     # fetch selection text
		if index:
			self.remove(index, self.listbox)		# actually remove the rpm   


#************************************************************************************
# Go here from menu choice "Open Directory".
# getdir()
#       - opens a file dialog box
#       - returns "rpmlist" which is reloaded in the main listbox
#       - disable the "Remove" button
#       - enable the "Install" button
#       - restores the "Query" and "List Files" button to their original function
#------------------------------------------------------------------------------------ 
	def getdir(self):
		mymode = self.mode		# use for temp place holder so we can hide the listbox
		if self.mode == FTP:
			self.con.uid = None		# clear any uid and passwd from an FTP session
			self.con.passwd = None

		self.mode = DIRECTORY
		root = Tk()
		root.withdraw()
		dir = GetDir(root)
		self.newdir = dir.go()		# get new directory

		if self.newdir != None:
			rlist = os.listdir(self.newdir)
			rpmlist = []
			for i in rlist:
				if i[-4:] == '.rpm':
					rpmlist.append(i)
			rpmlist.sort()
			self.label.config(text='Uninstalled rpms from:')
			self.groupLabel.config(text='Package Name')
			self.labelpath.config(text=self.newdir[:-2])
			self.reloadRpmBox(rpmlist)
			self.removeButton.config(state='disabled')
			self.installButton.config(state='normal')
		else:		# do this if the list is empty, ie "Cancel" pressed
			self.getinstalled()

		if mymode == GROUP:
			Pack.forget(self.groupbox)	# Hide the listbox
			

#************************************************************************************
# Go here from menu choice "List Installed".
# getinstalled()
#       - gets a list of installed rpm's
#       - returns "rpmlist" which is reloaded in the main listbox
#       - enable the "Remove" button
#       - disable the "Install" button
#------------------------------------------------------------------------------------ 
	def getinstalled(self):

		if self.mode == FTP:
			self.reloadRpmBox(['\nClosing FTP connection'])
			self.con.uid = None		# clear any uid and passwd from an FTP session
			self.con.passwd = None
			self.mlbframe.config(cursor='watch')		
			self.master.update()
			try:
				self.con.site.quit()	# close the connection
			except SyntaxError:
				i = 0				
			self.mlbframe.config(cursor='')		
			self.master.update()
		
		self.newdir = None		# reset this to none for handleList() etc
		rpmlist = os.popen('rpm -qa | sort', 'r').readlines()
		self.label.config(text='Installed rpms')
		self.groupLabel.config(text='Package Name')
		self.labelpath.config(text='')		
		self.reloadRpmBox(rpmlist)		
		self.removeButton.config(state='normal')
		self.installButton.config(state='disabled')
		if self.mode == GROUP:
			Pack.forget(self.groupbox)	# Hide the listbox
		self.mode = INSTALLED	

#************************************************************************************
# Go here from menu choice "Groups".  Won't display in DIR or FTP mode
# getgroup(item)
#       - gets a list of installed rpm's in a (item) group
#       - returns "rpmlist" which is reloaded in the main listbox
#       - enable the "Remove" button
#       - disable the "Install" button
#------------------------------------------------------------------------------------ 
	def getgroup(self, group):

		if self.mode == FTP:
			self.reloadRpmBox(['\nClosing FTP connection'])
			self.con.uid = None		# clear any uid and passwd from an FTP session
			self.con.passwd = None
			self.mlbframe.config(cursor='watch')		
			self.master.update()
			try:
				self.con.site.quit()	# close the connection
			except SyntaxError:
				i = 0				
			self.mlbframe.config(cursor='')		
			self.master.update()
		
		self.mode = GROUP
		
		self.newdir = None		# reset this to none for handleList() etc
		querystring = 'rpm -qa --queryformat "%{GROUP} %{NAME}-%{VERSION}-%{RELEASE}\n" |grep ^'+group+' | sort'
		rpmlist = os.popen(querystring, 'r').readlines()


		self.groupbox.pack()	# show the group listbox
		self.label.config(text='Installed rpms in '+group)
		self.labelpath.config(text='')		
		self.groupLabel.config(text='Package Name                                                  Subgroup Name')

		p = []
		g = []
		
		for package in rpmlist:    # add to list-box
			if package[-1:] == '\n':	# remove trailing newlines
				package = package[:-1]

			s = split(package)	#s[0] is group, #s[1] is package
			g.append(s[0])
			p.append(s[1])
				
		self.reloadRpmBox(p)

		self.groupbox.delete(0, END)
		pos = 0
		for i in g:
			i = i[len(group)+1:]
			self.groupbox.insert(pos, i)
			pos = pos + 1
							
		self.removeButton.config(state='normal')
		self.installButton.config(state='disabled')


#************************************************************************************
# Go here from menu choice "Open FTP Site".
# getftpsite()
#       - opens a listbox with saved ftp sites
#       - returns "rpmlist" which is reloaded in the main listbox
#       - disable the "Remove" button
#       - enable the "Install" button
#------------------------------------------------------------------------------------ 
	def getftpsite(self):
		root = Tk()					# do window stuff
		root.withdraw()	
		self.con = GetFtp(root)		# open instance
		rpmlist = self.con.go()		# get list of rpms at site/dir

		if self.mode == GROUP:
			Pack.forget(self.groupbox)
			
		if rpmlist != None:
			self.label.config(text='Uninstalled rpms from:')
			self.groupLabel.config(text='Package Name')
			self.labelpath.config(text=self.con.connected_site+self.con.ftpdir)
			self.reloadRpmBox(rpmlist)
			self.mode = FTP								# tell handle functions who called them
			self.removeButton.config(state='disabled')
			self.installButton.config(state='normal')
		if rpmlist == [] or rpmlist == None:		# do this if the list is empty, ie "Cancel" pressed
			self.getinstalled()


	def getlogfile(self):
		if self.ConfigUseLog.get():
			listfile('/var/lib/rpm/xrpm-use.log')
		else:
			attention('Log file option is not enabled!')
		
		
#************************************************************************************
# called from many places
# reloadRpmBox(list)
#       - reloads the main listbox with a new set of rpms (list).
#------------------------------------------------------------------------------------
	def reloadRpmBox(self, list):
		self.listbox.delete(0, END)
		pos = 0
		for package in list:    # add to list-box
			if package[-1:] == '\n':
				package = package[:-1]
			self.listbox.insert(pos, os.path.basename(package))
			pos = pos + 1

		
	def quit(self):
		sys.exit(1)


# **************************** Main **********************************
if __name__ == '__main__': Xrpm().mainloop()
